package com.persagy.dptool;

import com.persagy.dptool.dialog.HelpDialog;
import com.persagy.dptool.dialog.MessageDialog;
import com.persagy.dptool.dto.PhysicalObject;
import javafx.concurrent.Task;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.*;
import javafx.scene.layout.BorderPane;
import javafx.stage.DirectoryChooser;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.net.URL;
import java.util.*;

public class PrimaryController implements Initializable {
    private static final Logger log = LoggerFactory.getLogger(PrimaryController.class);

    @FXML
    private BorderPane paneRoot;
    @FXML
    private TabPane paneTab;
    /** 配置文件目录 */
    @FXML
    public TextField txfDir;
    @FXML
    public TextField txfDirJson;
    @FXML
    public Button btnSelectDir;
    @FXML
    public TextField txfProject;
    @FXML
    public TextField txfProjectInput;
    @FXML
    public TextField txfDataPlatformInput;
    @FXML
    public TextField txfDirInput;
    @FXML
    public Button btnCheck;
    @FXML
    public Button btnCheckJson;
    /** 校验结果文本域 */
    @FXML
    public TextArea txaContent;
    @FXML
    public TextArea txaContentJson;
    @FXML
    public TextArea txaContentInput;
    @FXML
    public ProgressIndicator piState;
    /** 底部状态栏 */
    @FXML
    public Label lblState;

    @Override
    public void initialize(URL location, ResourceBundle resources) {
        piState.setProgress(-1);
        piState.setVisible(false);
        lblState.setText("");

        StringBuilder tableNameSBStr = new StringBuilder();
        for (int i = 0; i < ConstantData.tablesNameList.size(); i++) {
            tableNameSBStr.append("  " + (i+1) + ". " + ConstantData.tablesNameList.get(i) + "\n");
        }
        txaContentJson.setText("目前只能校验HBase数据库中，物理世界的以下数据表数据：\n" + tableNameSBStr);

        String checkResultFileName = "checkResult" + new Date().getTime() + ".txt";

    }

    /**
     * 选择配置文件目录
     * @param e
     */
    public void selectDir(ActionEvent e) {
        DirectoryChooser dirChooser = new DirectoryChooser();
        dirChooser.setTitle("选择配置文件目录");

        File folder = dirChooser.showDialog(paneRoot.getScene().getWindow());

        if(null != folder) {
            txfDir.setText(folder.getAbsolutePath());
        }
    }

    /**
     * 选择配置文件目录
     * @param e
     */
    public void selectDirJson(ActionEvent e) {
        DirectoryChooser dirChooser = new DirectoryChooser();
        dirChooser.setTitle("选择json文件目录");

        File folder = dirChooser.showDialog(paneRoot.getScene().getWindow());

        if(null != folder) {
            txfDirJson.setText(folder.getAbsolutePath());
        }
    }

    /**
     * 选择配置文件目录
     * @param e
     */
    public void selectDirJsonInput(ActionEvent e) {
        DirectoryChooser dirChooser = new DirectoryChooser();
        dirChooser.setTitle("选择json文件目录");

        File folder = dirChooser.showDialog(paneRoot.getScene().getWindow());

        if(null != folder) {
            txfDirInput.setText(folder.getAbsolutePath());
        }
    }

    public void inputJsonFile(ActionEvent e) {
        txaContentInput.setText("");
        String projectId = txfProjectInput.getText();
        PhysicalObject physicalObject = null;
        if(projectId != null && projectId.trim().length() == 12 && projectId.startsWith("Pj")) {
            physicalObject = new PhysicalObject(projectId, projectId.substring(2));
        }else {
            txaContentInput.setText("项目id不合法！");
            txfProject.requestFocus();
            return;
        }

        String dataPlatformService = txfDataPlatformInput.getText();
        if(null == dataPlatformService || dataPlatformService.trim().length() < 20) {
            txaContentInput.setText("数据平台地址不合法！");
            txfDataPlatformInput.requestFocus();
            return;
        }else if(!CommonUtil.dataPlatformConnectTest(dataPlatformService)) {
            txaContentInput.setText("数据平台服务访问不通！请确保服务地址填写正确，且保证服务可访问。");
            txfDataPlatformInput.requestFocus();
            return;
        }else {
            physicalObject.dataPlatformUrl = dataPlatformService;
        }

        File jsonFileDir = null;
        String filePath = txfDirInput.getText();

        if(null != filePath) {
            filePath = filePath.trim();
            jsonFileDir = new File(filePath);
            if(!jsonFileDir.exists() || !jsonFileDir.isDirectory()) {
                txaContentInput.setText("目录地址不合法！");
                txfDirInput.requestFocus();
                return;
            }
        }else {
            txaContentInput.setText("目录地址不合法！");
        }

        LinkedHashMap<String, File> fileMap = new LinkedHashMap<>();

        Map<String, File> realFileMap = new HashMap<>();
        for(File file : jsonFileDir.listFiles()) {
            if(file.isFile() && file.getName().endsWith(ConstantData.tablesFileType)) {
                realFileMap.put(file.getName(), file);
            }
        }

        for(String tableName : ConstantData.tablesNameList) {
            String realFileName = tableName+ConstantData.tablesFileType;
            if(realFileMap.containsKey(realFileName)) {
                fileMap.put(tableName, realFileMap.get(realFileName));
            }
        }

        if(fileMap.isEmpty() || fileMap.size() != 9) {
            StringBuilder errorTips = new StringBuilder("未找到完备、合法的数据表json文件！至少包含以下数据表文件：\n");
            for(int i=0; i<9; i++) {
                errorTips.append(" " + (i+1) + ". " + ConstantData.tablesNameList.get(i) + "\n");
            }
            txaContentInput.setText(errorTips.toString());
            return;
        }

        Task<Boolean> task = TaskFactory.jsonFileInput(this, jsonFileDir, physicalObject);
        new Thread(task).start();
    }

    /**
     * 检查数据平台配置文件
     * @param e
     */
    public void checkConfig(ActionEvent e) {
        File configFile = getFile(txfDir.getText(), "config.properties");
        File propertyFile = getFile(txfDir.getText(), "property");

        if(null == configFile && propertyFile == null) {
            txaContent.setText("未找到需要校验的config.properties文件和property数据字典文件夹，请确保所选目录是否正确。");
            return;
        }

        Task<Boolean> task = TaskFactory.checkConfig(this, configFile, propertyFile);
        new Thread(task).start();
    }

    /**
     * 检查物理世界数据表json文件
     * @param e
     */
    public void checkJsonFile(ActionEvent e) {
        txaContentJson.setText("");
        String projectId = txfProject.getText();
        PhysicalObject physicalObject = null;
        if(projectId != null && projectId.trim().length() == 12 && projectId.startsWith("Pj")) {
            physicalObject = new PhysicalObject(projectId, projectId.substring(2));
        }else {
            txaContentJson.setText("项目id不合法！");
            txfProject.requestFocus();
            return;
        }

        File jsonFileDir = null;
        String filePath = txfDirJson.getText();

        if(null != filePath) {
            filePath = filePath.trim();
            jsonFileDir = new File(filePath);
            if(!jsonFileDir.exists() || jsonFileDir.isFile()) {
                txaContentJson.setText("目录地址不合法！");
                txfDirJson.requestFocus();
                return;
            }
        }else {
            txaContentJson.setText("目录地址不合法！");
        }

        log.info("【物理世界数据表json文件】----校验----开始----");
        Task<Boolean> task = TaskFactory.checkTableJsonFiles(this, physicalObject, jsonFileDir);
        new Thread(task).start();
    }

    /**
     * 根据文件路径获取文件实例
     * @param dirPath
     * @return
     */
    private File getFile(String dirPath, String fileName) {
        File file = null;

        if(dirPath != null) {
            dirPath = dirPath.trim() + File.separator + fileName;
            file = new File(dirPath);
            if(!file.exists()) {
                file = null;
            }
        }
        return file;
    }

    /**
     * 设置tab页签中的控件是否可用
     * @param disable true-可用
     * @param typeIndex 从1开始
     */
    public void setDisable(boolean disable, int typeIndex) {
        if(1 == typeIndex) {
            paneTab.setDisable(disable);
        }else if(2 == typeIndex) {
            paneTab.setDisable(disable);
        }else if(3 == typeIndex) {
            paneTab.setDisable(disable);
        }
    }

    public void showHelp(ActionEvent e) {
        new HelpDialog(paneRoot.getScene().getWindow().getX() + 20, paneRoot.getScene().getWindow().getY() + 20).showAndWait();
    }

    public void showAbout(ActionEvent e) {
        String aboutInfo = "Version：1.0\nJDK：1.8\nAuthor：cuixubin@persagy.com";
        new MessageDialog("About", aboutInfo, 300, 120, paneRoot.getScene().getWindow().getX() + 20, paneRoot.getScene().getWindow().getY() + 20).showAndWait();
    }
}
